<?php
namespace wcf\acp\form;
use wcf\form\AbstractForm;
use wcf\data\didyouknow\DidYouKnowEntryAction;
use wcf\system\exception\SystemException;
use wcf\system\exception\UserInputException;
use wcf\system\WCF;
use wcf\system\WCFACP;
use wcf\util\XML;
use wcf\util\StringUtil;

/**
 * Shows the did you know import and export form.
 *
 * @author	Jonathan Frick <defense>
 * @copyright	2009-2012 WBB-World
 * @license	Creative Commons BY-SA 3.0 <http://creativecommons.org/licenses/by-sa/3.0/de/legalcode>
 * @package	de.wbb-world.wcf.didyouknow
 * @subpackage	acp.form
 * @category 	Community Framework
 */
class DidYouKnowImportAndExportForm extends AbstractForm {
	/**
	 * @see wcf\page\AbstractPage::$templateName
	 */
	public $templateName = 'didYouKnowImport';

	/**
	 * @see wcf\page\AbstractPage::$neededPermissions
	 */
	public $neededPermissions = array('admin.didYouKnow.canAddEntry','admin.didYouKnow.canEditEntry','admin.didYouKnow.canDeleteEntry');

	/**
	 * @see wcf\acp\form\AbstractForm::$activeMenuItem
	 */
	public $activeMenuItem = 'wcf.acp.menu.link.didyouknow.importAndExport';

	/**
	 * upload file data
	 * @var array
	 */
	public $entryImport = null;

	/**
	 * detected file type
	 * @var string
	 */
	public $entryImportType = 'xml';

	/**
	 * allows an import with an old WCF1.1-export as source
	 * @var boolean
	 */
	public $enableCompatibilityMode = false;
	
	/**
	 * truncate before import
	 * @var boolean
	 */
	public $truncate = false;

	
	/**
	 * list of entrys
	 * @var array
	 */
	public $entrys = array();

	/**
	 * @see Form::readFormParameters()
	 */
	public function readFormParameters() {
		parent::readFormParameters();

		if (isset($_FILES['entryImport'])) $this->entryImport = $_FILES['entryImport'];
		if (isset($_POST['entryImportType'])) $this->entryImportType = $_POST['entryImportType'];
		if (isset($_POST['truncate'])) $this->truncate = $_POST['truncate'];
	}
	
	/**
	 * @see wcf\form\IForm::validate()
	 */
	public function validate() {
		parent::validate();

		// upload
		if ($this->entryImport && $this->entryImport['error'] != 4) {
			if ($this->entryImport['error'] != 0) {
				throw new UserInputException('entryImport', 'uploadFailed');
			}

			// detect import type
	      	if ($this->entryImportType == "detect") {
	      		$importFile = file($this->entryImport['tmp_name']);

				// get extension
				$extensionArray = explode(".", $this->entryImport['name']);
				$extension = array_pop($extensionArray);

				// find type
				if (count($importFile) > 1 && preg_match('/<\?xml version(.*)>/i',$importFile[0]) && preg_match('/<didyouknow(.*)>/i',$importFile[1]) ) {
					$this->entryImportType = "xml";
				} elseif ($extension == "txt") {
					$this->entryImportType = "text";
				} else {
					throw new UserInputException('entryImportType', 'detectionFailed');
				}
     		}
			elseif ($this->entryImportType == 'compatibility'){
				$this->entryImportType = 'xml';
				$this->enableCompatibilityMode = true;
			}

			try {
				switch ($this->entryImportType) {
					case 'xml':	
						$this->importFromXML ();
						break;
					case 'text':
						$this->importFromTXT ();
						break;
					default:
						throw new UserInputException('entryImportType', 'invalid');
				}
			}
			catch (SystemException $e) {
				throw new UserInputException('entryImport', 'importFailed');
			}
		}
		else {
			throw new UserInputException('entryImport', 'importFailed');
		}
	}

	/**
	 * Imports the entrys using a valid DidYouKnow-XML file.
	 */
	public function importFromXML () {
		$xml = new XML();
		$xml->load($this->entryImport['tmp_name']);
		//$xml->validate();
		$xpath = $xml->xpath();
		
		if ($this->enableCompatibilityMode) {
			$entrys = $xpath->query('/ns:didyouknow/ns:data');
		}
		else {
			$entrys = $xpath->query('/ns:didyouknow/ns:entrys/ns:entry');
		}
		foreach ($entrys as $entry) {
			// default values
			$values = array(
				'message' => '',
				'isDisabled' => 0,
				'enableHtml' => 0,
				'enableSmilies' => 1,
				'enableBBCodes' => 1,
				'priority' => 1);
			
			// fetch child elements
			$items = $xpath->query('child::*', $entry);
			foreach ($items as $item) {
				switch ($item->tagName) {
					case 'message':
						// check for i18n value
						if ($item->getAttribute('languageCode')) {
							if (!is_array($values['message'])) $values['message'] = array();
							$values['message'][$item->getAttribute('languageCode')] = $item->nodeValue;
						} 
						else {
							$values['message'] = $item->nodeValue;
						}
						break;
					case 'text':
						$values['text'] = $item->nodeValue;  // Compatibility for WCF1.1-Version
						break;
					case 'isDisabled':
						$values['isDisabled'] = $item->nodeValue;
						break;
					case 'enable':
						$values['isDisabled'] = !((int) $item->nodeValue); // Compatibility for WCF1.1-Version
						break;
					case 'priority':
						$values['priority'] = (int) $item->nodeValue;
						break;
					case 'enableSmilies':
					    $values['enableSmilies'] = $item->nodeValue;
						break;
					case 'enableHtml':
						$values['enableHtml'] = $item->nodeValue;
						break;
					case 'enableBBCodes':
				        $values['enableBBCodes'] = $item->nodeValue;
						break;
				}
			}
			
			$this->entrys[] = $values;
		}
		
		if (!count($this->entrys)) throw new UserInputException('entryImportType', 'invalid');
	}

	/**
	 * Imports the entrys using a text file.
	 */
	public function importFromTXT () {
        $file = file($this->entryImport['tmp_name']);

        foreach ($file as $line) {
            if (!empty($line)) {
				if (!StringUtil::isUTF8($line)) $line = utf8_encode ($line);
				$this->entrys[] = array(
					'message' => $line,
					'isDisabled' => 0,
					'enableHtml' => 0,
					'enableSmilies' => 1,
					'enableBBCodes' => 1,
					'priority' => 1
				);
            }
        }
		
		if (!count($this->entrys)) throw new UserInputException('entryImportType', 'invalid');
	}

	/**
	 * @see Form::save()
	 */
	public function save() {
		parent::save();

		// truncate if selected
		if ($this->truncate) {
			$sql = "TRUNCATE TABLE wcf".WCF_N."_didyouknow;";
			$statement = WCF::getDB()->prepareStatement($sql);
			$statement->execute();
		}

		// save
		$this->objectAction = new DidYouKnowEntryAction(array(), 'import', array('data' => $this->entrys));
		$this->objectAction->executeAction();
		$this->saved();

		// show success
		WCF::getTPL()->assign(array(
			'success' => true,
			'importedCount' => count($this->entrys)
		));
	}
}